from dataclasses import dataclass, field


@dataclass
class Finding:
    start: int
    end: int
    word: str
    rule: str
    section: str = ""
    description: str = ""
    weight: int = 100
    style: str = "red"
    tab: str = "red"
    source: str = "fast"


@dataclass
class ProofreadResult:
    score: float
    score_color: str
    text: str
    words: int
    sentences: int
    findings: list[Finding]
    stopwords: int
    summary: dict = field(default_factory=dict)
